#!/usr/bin/env python3

import os
import sys
import argparse
import re
import shutil
import subprocess
import logging
import json

import concurrent.futures

from cvehound import CVEhound
from cvehound.util import *
from cvehound.exception import UnsupportedVersion
from cvehound.cwe import CWE

def check_config(config):
    valid_config_options = {
        'kernel',
        'cve',
        'exclude',
        'exploit',
        'verbose',
        'cwe',
        'files',
        'ignore_files',
        'kernel_config'
        'check_strict',
        'report',
        'all_files',
        'metadata',
    }
    diff = set(config.keys()) - valid_config_options
    if diff:
        print("Unknown config options: " + ','.join(diff), file=sys.stderr)
        sys.exit(1)

def main(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(
        prog='cvehound',
        description='A tool to check linux kernel sources dump for known CVEs',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument('--config', metavar='FILE',
                        help='cvehound.ini config file (default: /etc/cvehound.ini or $HOME/.config/cvehound.ini')
    parser.add_argument('--kernel', '-k', metavar='DIR',
                        help='linux kernel sources dir')
    parser.add_argument('--list', action='store_true',
                        help="list all known CVEs and exit")
    parser.add_argument('--cve', '-c', nargs='+', default=['assigned'],
                        help='list of cve identifiers (groups: [all, assigned, disputed])')
    parser.add_argument('--exclude', '-x', nargs='+', default=[], metavar='CVE',
                        help='list of cve identifiers or a file with them to exclude CVEs from check')
    parser.add_argument('-v', '--verbose', action='count', default=0,
                        help='increase output verbosity')
    parser.add_argument('--exploit', '-e', action='store_true',
                        help='check only for CVEs with exploits')
    parser.add_argument('--cwe', nargs='+', default=[], type=int,
                        help='check only for CWE-ids')
    parser.add_argument('--files', nargs='+', default=[], metavar='PATH',
                        help='check only files (e.g. kernel drivers/block/floppy.c arch/x86)')
    parser.add_argument('--ignore-files', nargs='+', default=[], metavar='PATH',
                        help='exclude kernel files from check (e.g. kernel/bpf)')
    parser.add_argument('--kernel-config', nargs='?', const='-', metavar='.config',
                        help='check kernel config')
    parser.add_argument('--check-strict', action='store_true',
                        help='output only CVEs enabled in .config')
    parser.add_argument('--report', nargs='?', const='report.json',
                        help='output report with found CVEs')
    parser.add_argument('--all-files', action='store_true',
                        help="don't use files hint from cocci rules")
    parser.add_argument('--metadata', metavar='PATH',
                        help="Path to non-standard location of kernel_cves.json.gz")
    parser.add_argument('--version', action='version', version=get_cvehound_version())
    cmdargs = parser.parse_args()

    if cmdargs.list:
        (all_rules, _, _) = get_rule_cves()
        print("\n".join(sorted(all_rules)))
        sys.exit(0)

    config_args = {}
    try:
        if cmdargs.config:
            config_args = parse_config(cmdargs.config)
        else:
            if os.path.isfile('/etc/cvehound.ini'):
                config_args = parse_config('/etc/cvehound.ini')
            if 'HOME' in os.environ:
                home_config_path = os.path.join(os.environ['HOME'], '.config', 'cvehound.ini')
                if os.path.isfile(home_config_path):
                    config_args.update(parse_config(home_config_path))
    except Exception as err:
        print("Can't parse config file:", err, file=sys.stderr)
        sys.exit(1)
    check_config(config_args)
    args = config_args
    cmdargs = vars(cmdargs)
    for arg in cmdargs.keys():
        if cmdargs[arg] != parser.get_default(arg):
            args[arg] = cmdargs[arg]
        elif arg not in args:
            args[arg] = cmdargs[arg]

    if not args['kernel']:
        parser.print_usage()
        print("cvehound: error: the following arguments are required: --kernel/-k", file=sys.stderr)
        sys.exit(1)

    if args['metadata']:
        if not os.path.isfile(args['metadata']):
            print("Can't find metadata file", args['metadata'], file=sys.stderr)
            sys.exit(1)
        if not args['metadata'].endswith('.gz'):
            print("Metadata file", args['metadata'], "is not the gz archive", file=sys.stderr)
            sys.exit(1)

    if not all(os.path.isfile(os.path.join(args['kernel'], f)) for f in
               ['Makefile']):
        print(args['kernel'], "isn't a kernel directory", file=sys.stderr)
        sys.exit(1)

    if args['kernel_config'] == '-':
        config = os.path.normpath(os.path.join(args['kernel'], '.config'))
        if os.path.isfile(config):
            args['kernel_config'] = config
    else:
        if args['kernel_config'] and not os.path.isfile(args['kernel_config']):
            print("Can't find config file", args['kernel_config'], file=sys.stderr)
            sys.exit(1)

    if args['kernel_config'] and args['verbose'] == 0:
        args['verbose'] = 1

    if args['check_strict']:
        if not args['kernel_config']:
            print('Please, use --check-strict with --kernel-config', file=sys.stderr)
            sys.exit(1)

    if not shutil.which('spatch'):
        print("coccinelle is not installed", file=sys.stderr)
        sys.exit(1)

    loglevel = logging.WARNING
    if args['verbose'] > 1:
        loglevel = logging.DEBUG
    elif args['verbose'] > 0:
        loglevel = logging.INFO
    logging.basicConfig(level=loglevel, format='%(message)s')

    config_info = {}
    if args['kernel_config'] and args['kernel_config'] != '-':
        config_info = get_config_data(args['kernel_config'])

    hound = CVEhound(args['kernel'], args['metadata'], args['kernel_config'],
                     args['check_strict'], config_info.get('arch', 'x86'))

    cve_id = re.compile(r'^CVE-\d{4}-\d{4,7}$')
    if args['cve'] == ['all']:
        args['cve'] = hound.get_all_cves()
    elif args['cve'] == ['assigned']:
        args['cve'] = hound.get_assigned_cves()
    elif args['cve'] == ['disputed']:
        args['cve'] = hound.get_disputed_cves()
    else:
        known_cves = hound.get_all_cves()
        for i, cve in enumerate(args['cve']):
            if not cve.startswith('CVE-'):
                cve = 'CVE-' + cve
                args['cve'][i] = cve
            if not cve_id.match(cve):
                print('Wrong CVE-ID:', cve, file=sys.stderr)
                sys.exit(1)
            if cve not in known_cves:
                print('Unknown CVE:', cve, file=sys.stderr)
                sys.exit(1)

    for file in args['exclude']:
        if os.path.exists(file):
            args['exclude'].remove(file)
            with open(file, 'rt', encoding='utf-8') as fh:
                for line in fh:
                    line = line.strip()
                    if line == '' or line.startswith('#'):
                        continue
                    if not line.startswith('CVE-'):
                        line = 'CVE-' + line
                    if not cve_id.match(line):
                        print('Wrong CVE-ID:', line, 'in file', file, file=sys.stderr)
                        sys.exit(1)
                    args['exclude'].append(line)

    for i, cve in enumerate(args['exclude']):
        if not cve.startswith('CVE-'):
            cve = 'CVE-' + cve
            args['exclude'][i] = cve
        if not cve_id.match(cve):
            print('Wrong CVE-ID:', cve, file=sys.stderr)
            sys.exit(1)

    if args['all_files'] and args['files']:
        print('--files filter and --all-files are not compatible', file=sys.stderr)
        sys.exit(1)
    if args['all_files'] and args['ignore_files']:
        print('--ignore-files filter and --all-files are not compatible', file=sys.stderr)
        sys.exit(1)
    for f in [*args['files'], *args['ignore_files']]:
        path = re.compile(r'^[_a-zA-Z-./0-9]+$')
        if not path.match(f):
            print('Wrong file filter:', f, file=sys.stderr)
            sys.exit(1)

    filter_cwes = frozenset(args['cwe'])
    cves = []
    for cve in args['cve']:
        if cve in args['exclude']:
            continue
        if args['exploit'] and not hound.get_cve_exploit(cve):
            continue
        if args['cwe']:
            rule_cwe_desc = hound.get_cve_cwe(cve)
            if not rule_cwe_desc:
                continue
            rule_cwes = frozenset(CWE[rule_cwe_desc])
            if not (rule_cwes & filter_cwes):
                continue
        if args['files']:
            add = False
            for rulefile in hound.get_rule_files(cve):
                if any(map(lambda x: rulefile.startswith(x), args['files'])):
                    add = True
                    break
            if not add:
                continue
        if args['ignore_files']:
            ignore = True
            for rulefile in hound.get_rule_files(cve):
                if all(map(lambda x: not rulefile.startswith(x) and not rulefile.endswith('.h'), args['ignore_files'])):
                    ignore = False
                    break
            if ignore:
                continue
        cves.append(cve)

    args['files'].sort()
    args['ignore_files'].sort()
    args['cwe'].sort()
    args['cve'] = sorted(cves)

    report = { 'args': {}, 'kernel': {}, 'config': {}, 'tools': {}, 'results': {}}
    report['args']['cve'] = args['cve']
    report['args']['kernel'] = args['kernel']
    report['args']['config'] = args['kernel_config']
    report['args']['only_cwe'] = args['cwe']
    report['args']['only_files'] = args['files']
    report['args']['all_files'] = args['all_files']
    report['args']['check_strict'] = args['check_strict']
    report['kernel'] = get_kernel_version(args['kernel'])
    if args['kernel_config'] != '-':
        report['config'] = config_info
    report['tools']['cvehound'] = get_cvehound_version()
    report['tools']['spatch'] = '.'.join(list(str(get_spatch_version())))

    with concurrent.futures.ProcessPoolExecutor(max_workers=os.cpu_count()) as executor:
        future_to_cve = { executor.submit(hound.check_cve, cve, args['all_files']): cve
                          for cve in args['cve'] }

        for future in concurrent.futures.as_completed(future_to_cve):
            cve = future_to_cve[future]
            try:
                result = future.result()
                if result:
                    report['results'][cve] = result
            except subprocess.CalledProcessError as e:
                logging.error('Failed to run: ' + ' '.join(e.cmd) + '\nError: ' + e.stderr)
            except UnsupportedVersion as err:
                logging.error('Skipping: ' + err.cve + ' requires spatch >= ' + err.rule_version)

    if args['report']:
        with open(args['report'], 'wt', encoding='utf-8') as fh:
            json.dump(report, fh, indent=4, sort_keys=True)
        print('Report saved to:', args['report'])

if __name__ == '__main__':
    main(sys.argv[1:])



""" Base classes for parser functionality extensions. """

# Copyright (C) 2014-2015 Andreas Ruprecht <andreas.ruprecht@fau.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

class InitClass(object):
    """ Base class for initializer objects for the parser. """

    def __init__(self, model, arch):
        """ Constructor for InitClass, takes model and arch parameters."""
        self.model = model
        self.arch = arch

    def get_file_for_subdirectory(self, directory):
        """ This method must implement the selection and precedence rules
        for files in a given subdirectory. """
        sys.exit("E: get_file_for_subdirectory not implemented!")

    def process(self, parser, args, dirs_to_process):
        """ Initialize data structures that are needed across runs, such as
        the directories which will be processed or any global variables."""
        pass


class BeforePass(object):
    """ Base class for functionality run inside a Makefile but before
    any lines are read from the file. """

    def __init__(self, model, arch):
        """ Constructor for BeforePass, takes model and arch parameters."""
        self.model = model
        self.arch = arch

    def process(self, parser, basepath):
        """ This function is executed before the main processing loop to
        initialize local variables or other helper structures."""
        pass


class DuringPass(object):
    """ Base class for functionality exerted on every line in the Makefile."""

    def __init__(self, model, arch):
        """ Constructor for DuringPass, takes model and arch parameters."""
        self.model = model
        self.arch = arch

    def process(self, parser, line, basepath):
        """ Processing function. Returns true to signal successful handling
        so other DuringPass classes will not consider this line."""
        pass


class AfterPass(object):
    """ Base class for functionality after a file has been parsed. """

    def __init__(self, model, arch):
        """ Constructor for AfterPass, takes model and arch parameters."""
        self.model = model
        self.arch = arch

    def process(self, parser, path, condition_for_current_dir):
        """ This is called after the main processing loop has finished.
        Here, descending into subdirectories or expanding macros can be
        achieved."""
        pass


class BeforeExit(object):
    """ Base class for functionality being called right before the end of the
    parser process. """

    def __init__(self, model, arch):
        """ Constructor for BeforeExit """
        self.model = model
        self.arch = arch

    def process(self, parser):
        """ This is called after all base directories have been crawled and
        the parser is about to exit."""
        pass



#!/usr/bin/env python3

import os
import pytest
import tempfile
import psutil
from cvehound import CVEhound, get_rule_cves
from git import Repo
from subprocess import run

missing_backports = [
    ('CVE-2019-12455', 'stable/linux-4.19.y'), # Disputed
    ('CVE-2021-4037', 'stable/linux-4.19.y'),
    ('CVE-2022-0998', 'stable/linux-5.15.y'),
    ('CVE-2022-1789', 'stable/linux-4.19.y'),
    ('CVE-2022-1789', 'stable/linux-5.4.y'),
    ('CVE-2022-3061', 'stable/linux-4.19.y'),
    ('CVE-2022-3061', 'stable/linux-5.4.y'),
    ('CVE-2023-4133', 'stable/linux-4.19.y'),
    ('CVE-2023-4133', 'stable/linux-5.4.y'),
    ('CVE-2023-4133', 'stable/linux-5.10.y'),
    ('CVE-2023-4133', 'stable/linux-5.15.y'),
    ('CVE-2023-4133', 'stable/linux-6.1.y'),
    ('CVE-2023-23005', 'stable/linux-6.1.y'),
    ('CVE-2024-26595', 'stable/linux-4.19.y'),
    ('CVE-2024-26595', 'stable/linux-5.4.y'),
    ('CVE-2024-26595', 'stable/linux-5.10.y'),
    ('CVE-2024-26595', 'stable/linux-5.15.y'),
    ('CVE-2024-26595', 'stable/linux-6.1.y'),
]

def mount_tmpfs(target, req_mem_gb):
    if os.path.ismount(target):
        return True
    lines = []
    with open('/proc/meminfo') as fh:
        lines = fh.readlines()
    meminfo = dict((i.split()[0].rstrip(':'),int(i.split()[1])) for i in lines)
    av_mem_gb = int(meminfo['MemAvailable'] / 1024 ** 2)
    if av_mem_gb >= req_mem_gb + 1:
        ret = run(['sudo', '--non-interactive',
                   'mount', '-t', 'tmpfs', '-o', 'rw,noatime,nosuid,nodev,noexec,size=' + str(req_mem_gb) + 'G', 'tmpfs', target])
        return ret.returncode == 0
    else:
        return False

def mount_overlayfs(lower, upper, workdir, target):
    if os.path.ismount(target):
        return True
    ret = run(['sudo', '--non-interactive',
               'mount', '-t', 'overlay', '-o', 'rw,lowerdir=' + lower + ',upperdir=' + upper + ',workdir=' + workdir, 'overlay', target])
    return ret.returncode == 0

def umount(target):
    if os.path.ismount(target):
        run(['sudo', '--non-interactive', 'umount', target])

def pytest_addoption(parser):
    parser.addoption(
        '--cve',
        action='append',
        default=[],
        help='list of CVEs',
    )
    parser.addoption(
        '--branch',
        action='append',
        default=[],
        help='list of linux-stable branches to run tests on',
    )
    parser.addoption(
        '--runslow', action='store_true', default=False, help='run slow tests'
    )
    parser.addoption(
        '--runlkc', action='store_true', default=False, help='run lkc metadata tests'
    )
    parser.addoption(
        '--dir', action='store',
        default=os.path.join(os.path.dirname(os.path.realpath(__file__)), 'linux'),
        help='linux kernel sources dir'
    )

overlaydir = None
linux_mount = None
linux_repo = None
_cvehound = None
branches = []
cves = []

def pytest_configure(config):
    global overlaydir
    global linux_mount
    global linux_repo
    global _cvehound
    global branches
    global cves

    config.addinivalue_line('markers', 'slow: mark test as slow to run')
    config.addinivalue_line('markers', 'fast: fast tests that are duplicated by slow ones')
    config.addinivalue_line('markers', 'notbackported: mark test as failed')
    config.addinivalue_line('markers', 'ownfixes: mark test as failed')

    try:
        p = psutil.Process()
        p.nice(-100)
        p.ionice(psutil.IOPRIO_CLASS_RT, value=0)
    except:
        pass

    linux = config.getoption('dir')
    repo = None
    if os.path.isdir(os.path.join(linux, '.git')):
        repo = Repo(linux)
        repo.head.reset(index=True, working_tree=True)
        repo.git.clean('-f', '-x', '-d')
        repo.git.checkout('origin/master')
        try:
            repo.remotes.origin.fetch()
            repo.remotes.stable.fetch()
            repo.remotes.next.fetch()
        except:
            pass
    else:
        cwd = os.getcwd()
        os.makedirs(linux, exist_ok=True)
        os.chdir(linux)
        repo = Repo.clone_from('git://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git', '.')
        repo.create_remote('stable', 'git://git.kernel.org/pub/scm/linux/kernel/git/stable/linux.git')
        repo.create_remote('next', 'git://git.kernel.org/pub/scm/linux/kernel/git/next/linux-next.git')
        repo.remotes.stable.fetch()
        repo.remotes.next.fetch()
        os.chdir(cwd)

    overlaydir = tempfile.mkdtemp()
    linux_mount = tempfile.mkdtemp()
    if mount_tmpfs(overlaydir, 2):
        upperdir = os.path.join(overlaydir, 'upper')
        workdir = os.path.join(overlaydir, 'work')
        os.mkdir(upperdir)
        os.mkdir(workdir)
        mount_overlayfs(linux, upperdir, workdir, linux_mount)
        linux_repo = Repo(linux_mount)
    else:
        linux_repo = repo

    _cvehound = CVEhound(linux_repo.working_tree_dir)

    branches = config.getoption('branch')
    if not branches:
        branches = [
            'origin/master',
            'next/master',
            'stable/linux-6.6.y',
            'stable/linux-6.1.y',
            'stable/linux-5.15.y',
            'stable/linux-5.10.y',
            'stable/linux-5.4.y',
            'stable/linux-4.19.y',
        ]

    cves = config.getoption('cve')
    if not cves:
        (cves, _, _) = get_rule_cves()
        cves = cves.keys()

def pytest_unconfigure(config):
    umount(linux_mount)
    umount(overlaydir)
    os.rmdir(overlaydir)
    os.rmdir(linux_mount)

@pytest.fixture
def repo():
    return linux_repo

@pytest.fixture
def hound():
    return _cvehound

prev_branch = None
@pytest.fixture
def branch(request):
    global prev_branch
    if prev_branch != request.param:
        linux_repo.git.checkout('--force', request.param)
        prev_branch = request.param
    return request.param

def pytest_generate_tests(metafunc):
    if 'branch' in metafunc.fixturenames:
        metafunc.parametrize('branch', branches, indirect=True)

    if 'cve' in metafunc.fixturenames:
        metafunc.parametrize('cve', cves)

def pytest_collection_modifyitems(config, items):
    runslow = config.getoption('--runslow')
    runlkc = config.getoption('--runlkc')
    skip_slow = pytest.mark.skip(reason='need --runslow option to run')
    skip_fast = pytest.mark.skip(reason='slow tests cover these testcases')
    skip_lkc = pytest.mark.skip(reason='need --runlkc option to run')
    fail_notbackported = pytest.mark.xfail(reason='CVE not backported yet')
    for item in items:
        if not runslow and 'slow' in item.keywords:
            item.add_marker(skip_slow)
        if runslow and 'fast' in item.keywords:
            item.add_marker(skip_fast)
        if not runlkc and 'lkc' in item.keywords:
            item.add_marker(skip_lkc)
        if 'notbackported' in item.keywords:
            params = item.callspec.params
            mark = None
            for m in item.own_markers:
                if m.name == 'notbackported':
                    mark = m
                    break
            if (params['cve'], params['branch']) in mark.args[1]:
                item.add_marker(fail_notbackported)
        if 'ownfixes' in item.keywords:
            params = item.callspec.params
            mark = None
            for m in item.own_markers:
                if m.name == 'ownfixes':
                    mark = m
                    break
            for rec in mark.args[1]:
                if params['cve'] == rec[0]:
                    item.add_marker(pytest.mark.xfail(reason=rec[1]))



import os
import re

import cvehound.kbuildparse.base_classes as BaseClasses
import cvehound.kbuildparse.data_structures as DataStructures
import cvehound.kbuildparse.helper as Helper
import cvehound.kbuildparse.linux as Linux

class KbuildParser(object):
    """ Main class: parse Kbuild files recursively."""

    def __init__(self, model=None, arch='x86'):
        """ Initialize the parser. We need a model for _MODULE options."""
        self.model = model
        self.arch = arch
        self.local_vars = DataStructures.VariableStore()
        self.global_vars = DataStructures.VariableStore()
        self.init_class = Linux.LinuxInit(model, arch)
        self.before_pass = [Linux.LinuxBefore(model, arch)]
        self.during_pass = [
            Linux._00_LinuxDefinitions(model, arch),
            Linux._01_LinuxIf(model, arch),
            Linux._02_LinuxObjects(model, arch)
        ]
        self.output = Linux._03_LinuxOutput(model, arch)
        self.after_pass = [
            Linux._01_LinuxExpandMacros(model, arch),
            Linux._02_LinuxProcessSubdirectories(model, arch),
            self.output
        ]
        self.before_exit = []
        self.file_content_cache = {}

    def enter_new_symbolic_level(self):
        """ Get a fresh mapping for variables, save old mapping in nxt."""
        new_store = DataStructures.VariableStore()
        new_store.nxt = self.local_vars
        self.local_vars = new_store

    def leave_symbolic_level(self):
        """ Restore old mapping from local_vars.nxt."""
        assert self.local_vars.nxt is not None
        self.local_vars = self.local_vars.nxt

    def process_kbuild_or_makefile(self, path, conditions):
        """ Central processing function. Parse the file in @path which
        has preconditions @conditions. Processing is done by classes which
        have previously been gathered in corresponding lists."""

        if not os.path.isfile(path):
            return

        basepath = os.path.dirname(path)

        # Create new symbol table for local variables
        self.enter_new_symbolic_level()

        # Execute BeforePass subclass functions
        for processor in self.before_pass:
            processor.process(self, basepath)

        self.read_whole_file(path)

        # Main processing loop, iteration over file
        for line in self.file_content_cache[path]:
            # Execute DuringPass module functions
            for processor in self.during_pass:
                # As soon as one method returns True, continue with next line
                if processor.process(self, line, basepath):
                    break
        # End of main processing loop

        # Execute subclasses of AfterPass
        for processor in self.after_pass:
            processor.process(self, path, conditions)

        # Drop current symbol table
        self.leave_symbolic_level()

    def get_config(self):
        return self.output.config 

    # A bit of a hack to support ifdefs around variable additions in the
    # Makefiles included by drivers/gpu/drm/amd/amdgpu/Makefile - it uses
    # variables with numbers in them, which we purposefully do not capture
    # here; they are evaluated by the macro expansion while processing the
    # file.
    variable_name_regex = r"[A-Z_]+"
    def_regex = re.compile(r"([^\s:]*)\s*[\?:]?=\s*(.*)$")
    def_add_regex = re.compile(r"([^\s]*)\s*\+=\s*(.*)$")
    parseable_regex = re.compile(r"\$\((.*?) (.*?)\s*,\s*(.*)\)$")

    def process_addprefix(self, prefix, targets):
        l = []
        for t in targets.split():
            l.append(prefix + t)
        return " ".join(l)

    def process_addsuffix(self, suffix, targets):
        l = []
        for t in targets.split():
            l.append(t + suffix)
        return " ".join(l)

    def replace_variables(self, content, defs, srcpath):
        used_vars = re.findall(r"\$\((" + self.variable_name_regex + r")\)", content)
        content = re.sub(r"\$\(src\)", srcpath, content)
        for var in used_vars:
            if not var in defs:
                continue
            content = re.sub(r"\$\(" + var + r"\)", defs[var], content)
        return content

    def parse_replacements(self, content, defs):
        m = self.parseable_regex.match(content)
        if m:
            what, token, lst = m.groups()
            # recursive evaluation of argument list
            lst = self.parse_replacements(lst, defs)
            if what == 'addprefix':
                content = self.process_addprefix(token, lst)
            elif what == 'addsuffix':
                content = self.process_addsuffix(token, lst)
        return content

    def resolve(self, content, defs, srcpath="."):
        content = self.replace_variables(content, defs, srcpath)
        return self.parse_replacements(content, defs)

    def note_definition(self, line, defs):
        match = self.def_add_regex.match(line)
        if match:
            lhs, rhs = match.groups()
            resolved = self.resolve(rhs, defs)
            if re.match(self.variable_name_regex, lhs):
                if not lhs in defs:
                    defs[lhs] = ''
                defs[lhs] += ' ' + resolved
            line = '{} += {}'.format(lhs, resolved)
        else:
            match = self.def_regex.match(line)
            if match:
                lhs, rhs = match.groups()
                resolved = self.resolve(rhs, defs)
                if re.match(self.variable_name_regex, lhs):
                    defs[lhs] = resolved
                line = '{} := {}'.format(lhs, resolved)
        return line

    def read_whole_file(self, path):
        """ Read the content of the file in @path into the file_content_cache
        dictionary. Include statements are resolved on-the-fly (see comment in
        resolve_includes())."""
        defs = {}
        output = []
        with open(path, "r") as infile:
            dirname = os.path.dirname(path)
            while True:
                (good, line) = Helper.get_multiline_from_file(infile)
                if not good:
                    break

                line = self.note_definition(line, defs)

                inputs = self.resolve_includes(line, dirname, defs)
                output.extend(inputs)

        self.file_content_cache[path] = output

    def resolve_includes(self, line, srcpath, defs):
        """ If @line starts with "include", read all the lines in the included
        file. This is done recursively to treat recursive includes. The @srcpath
        parameter is needed to correctly resolve the $(src) variable in the
        included files (it needs to contain the path to the folder of the
        top-most including Makefile)."""

        if not line.startswith("include "):
            return [DataStructures.LineObject(line)]

        line = self.replace_variables(line, defs, srcpath)

        lines = []

        targets = [x.rstrip() for x in line.split()[1:]]
        for target in targets:
            if not os.path.isfile(target):
                target = os.path.join(srcpath, target)
                if not os.path.isfile(target):
                    continue

            with open(target, "r") as infile:
                while True:
                    (good, line) = Helper.get_multiline_from_file(infile)
                    if not good:
                        break
                    line = self.note_definition(line, defs)
                    lines.extend(self.resolve_includes(line, srcpath, defs))

        return lines




from collections.abc import Mapping

class Config(Mapping):

    def __init__(self, config):
        enabled = {}
        with open(config, 'rt', encoding='utf8') as fh:
            for line in fh:
                line = line.strip()
                if line.startswith('CONFIG_'):
                    enabled[line.split('=')[0]] = True
                elif line.startswith('# CONFIG_') and line.endswith('is not set'):
                    enabled[line.split(' ')[1]] = False
        self._storage = enabled

    def __getitem__(self, key):
        try:
            self._storage[key]
        except Exceptions:
            return False

    def __iter__(self):
        return iter(self._storage)

    def __len__(self):
        return len(self._storage)

    def get_mapping(self):
        return self._storage



""" Implementation of kbuildparse base classes for Linux."""

# Copyright (C) 2014-2015 Andreas Ruprecht <andreas.ruprecht@fau.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import collections
import logging
import os
import re

import cvehound.kbuildparse.base_classes as BaseClasses
import cvehound.kbuildparse.data_structures as DataStructures
import cvehound.kbuildparse.helper as Helper

# Helper functions in module
CONFIG_FORMAT = r"CONFIG_([A-Za-z0-9_-]+)"
REGEX_CONFIG = re.compile(CONFIG_FORMAT)
REGEX_IFNEQ = re.compile(r"\s*(ifneq|ifeq)\s+(.*)")
REGEX_IFNEQ_CONF = re.compile(r"\(\$\(" + CONFIG_FORMAT +
                              r"\),\s*(y|m|n|\s*)\s*\)")
REGEX_IFNDEF = re.compile(r"\s*(ifdef|ifndef)\s+(.*)")
REGEX_ENDIF = re.compile(r"\s*endif\s*")
REGEX_ELSE = re.compile(r"\s*else\s*")

def regex_ifneq_match(line, ifdef_condition, global_vars, model):
    """ Check if @line resembles a line describing a condition
    with if(n)eq. If so, update the condition in @ifdef_condition and
    return True, otherwise return False."""
    regex_match = REGEX_IFNEQ.match(line)
    if not regex_match:
        return False

    if global_vars["no_config_nesting"] > 0:
        global_vars.increment_variable("no_config_nesting")
    else:
        positive_keyword = (regex_match.group(1) != "ifneq")
        possible_conf = regex_match.group(2)
        conf_results = REGEX_IFNEQ_CONF.match(possible_conf)
        if conf_results:
            positive_comp = (conf_results.group(2) == "y" or
                             conf_results.group(2) == "m")
            config = conf_results.group(1)

            conf = Helper.get_config_string(config, model)
            if positive_keyword == positive_comp:
                ifdef_condition.add_condition(conf)
            else:
                ifdef_condition.add_condition("~" + conf)
        else:
            global_vars.increment_variable("no_config_nesting")
    return True

def regex_ifndef_match(line, ifdef_condition, global_vars, model):
    """ Check if @line resembles a line describing a condition
    with if(n)def. If so, update the condition in @ifdef_condition and
    return True, otherwise return False."""
    regex_match = REGEX_IFNDEF.match(line)
    if not regex_match:
        return False

    if global_vars["no_config_nesting"] > 0:
        global_vars.increment_variable("no_config_nesting")
    else:
        keyword = regex_match.group(1)
        rhs = regex_match.group(2)

        conf_results = REGEX_CONFIG.match(rhs)
        if conf_results:
            conf = Helper.get_config_string(conf_results.group(1),\
                                           model)
            if keyword == "ifdef":
                ifdef_condition.add_condition(conf)
            else:
                ifdef_condition.add_condition("~" + conf)
        else:   # Not parseable -> nesting level
            global_vars.increment_variable("no_config_nesting")
    return True

def regex_endif_match(line, ifdef_condition, global_vars):
    """ Check if @line resembles a line describing the end of a conditional
    block (endif). If so, update the condition in @ifdef_condition and
    return True, otherwise return False."""
    regex_match = REGEX_ENDIF.match(line)
    if not regex_match:
        return False

    if global_vars["no_config_nesting"] > 0:
        global_vars.decrement_variable("no_config_nesting")
    else:
        ifdef_condition.pop()
    return True

def regex_else_match(line, ifdef_condition, global_vars):
    """ Check if @line resembles a line describing an else.
    If so, update the condition in ifdef_condition and
    return True, otherwise return False."""
    regex_match = REGEX_ELSE.match(line)
    # Note that this regex can only match single "else" statements in one line,
    # constructs like "else ifeq $(...)" are not supported (at the time of
    # writing, this is not used at any relevant place in the Linux kernel).
    if not regex_match:
        return False

    if global_vars["no_config_nesting"] > 0:
        pass
    else:
        last = ifdef_condition.pop()
        if last.startswith("~"):
            ifdef_condition.add_condition(last[1:])
        else:
            ifdef_condition.add_condition("~" + last)
    return True

def update_if_condition(line, ifdef_condition, global_vars, local_vars, model):
    """ Update the @ifdef_condition with information from @line.
    If updating succeeded, return True. Otherwise signal skipping
    of the line by returning True if we're inside an unparseable block and
    False if the other logic should try to process it. """
    if regex_ifneq_match(line, ifdef_condition, global_vars, model) or \
            regex_ifndef_match(line, ifdef_condition, global_vars, model) or \
            regex_else_match(line, ifdef_condition, global_vars) or \
            regex_endif_match(line, ifdef_condition, global_vars):
        return True
    ret = global_vars["no_config_nesting"] > 0
    return ret


class LinuxInit(BaseClasses.InitClass):
    """ Init class for Linux."""

    # Arch specific regexes
    arch_spec_line = r"\s*(core|init|drivers|net|libs)-(y|\$\(" + \
                     CONFIG_FORMAT + r"\))\s*(=|\+=|:=)\s*(.*)"
    regex_arch_spec = re.compile(arch_spec_line)
    arm_line = r"\s*(machine|plat)-(y|\$\(" + CONFIG_FORMAT + \
               r"\))\s*(=|\+=|:=)\s*(.*)"
    regex_arm = re.compile(arm_line)
    include_line = r"include\s+(.*)"
    regex_include = re.compile(include_line)
    mips_platforms_line = r"platforms\s*\+=\s*(.*)"
    regex_mips_platforms = re.compile(mips_platforms_line)
    mips_plat_line = r"platform-\$\(" + CONFIG_FORMAT + r"\)\s*\+=\s*(.*)"
    regex_mips_plat = re.compile(mips_plat_line)

    def __init__(self, model, arch):
        """ Constructor for InitLinux, takes model and arch parameters."""
        super(LinuxInit, self).__init__(model, arch)

    def parse_architecture_regular(self, line, local_arch_dirs):
        """ Parse an architecture Makefile. This looks for any additional
        lists (core|init|drivers|net|libs) and saves the corresponding
        conditions for those directories."""
        regex_match = self.regex_arch_spec.match(line)
        if not regex_match:
            return False

        current_precondition = DataStructures.Precondition()
        config_item = regex_match.group(2)
        if config_item != "y":
            config_item = regex_match.group(3)
            condition = Helper.get_config_string(config_item, self.model)
            current_precondition.add_condition(condition)

        rhs = regex_match.group(5)

        matches = [x for x in re.split("\t| ", rhs) if x]

        for match in matches:
            if not os.path.isdir(match):
                continue

            if match[-1] != '/':
                match += '/'
            logging.debug("adding match " + match + " with " + \
                          config_item)
            local_arch_dirs[match].append(current_precondition[:])
        return True

    def parse_arm_architecture(self, line, local_arch_dirs):
        """ Parse the Makefile at arch/arm/ and look for
        machine-$() and plat-$() lists describing the subdirectories."""

        if self.arch != "arm":
            return False

        regex_match = self.regex_arm.match(line)
        if not regex_match:
            return False

        current_precondition = DataStructures.Precondition()
        lst = regex_match.group(1)
        config_item = regex_match.group(2)
        if config_item != "y":
            config_item = regex_match.group(3)
            condition = Helper.get_config_string(config_item, self.model)
            current_precondition.add_condition(condition)

        rhs = regex_match.group(5)

        matches = [x for x in re.split("\t| ", rhs) if x]

        for match in matches:
            fullpath = ""
            if lst == "machine":
                fullpath = "arch/arm/mach-" + match
            else:
                fullpath = "arch/arm/plat-" + match
            if not os.path.isdir(fullpath):
                continue

            if fullpath[-1] != '/':
                fullpath += '/'
            logging.debug("adding ARM match " + match + " with " + \
                          config_item)
            local_arch_dirs[fullpath].append(current_precondition[:])
        return True

    def parse_blackfin_architecture(self, line, local_arch_dirs):
        """ Parse the Makefile at arch/blackfin/ and look for any
        lines describing machine-$(CONFIG_XY) lists."""

        if self.arch != "blackfin":
            return False

        regex_match = self.regex_arm.match(line)
        if not regex_match:
            return False

        current_precondition = DataStructures.Precondition()
        lst = regex_match.group(1)
        config_item = regex_match.group(2)
        if config_item != "y":
            config_item = regex_match.group(3)
            condition = Helper.get_config_string(config_item, self.model)
            current_precondition.add_condition(condition)

        rhs = regex_match.group(5)

        matches = [x for x in re.split("\t| ", rhs) if x]

        for match in matches:
            if lst != "machine":    # Should never happen in blackfin
                logging.error("plat- list should not be present in blackfin")
                continue

            fullpath = "arch/blackfin/mach-" + match
            if os.path.isdir(fullpath):
                if fullpath[-1] != '/':
                    fullpath += '/'
                local_arch_dirs[fullpath].append(current_precondition[:])

            fullpath = "arch/blackfin/mach-" + match + "/boards/"
            if os.path.isdir(fullpath):
                if fullpath[-1] != '/':
                    fullpath += '/'
                local_arch_dirs[fullpath].append(current_precondition[:])
        return True

    def parse_mips_platform(self, path, local_arch_dirs):
        """ Parse a arch/mips/*/Platform file. Mips describes the dependencies
        there, which is why we need to parse this as well."""

        with open(path, "r") as infile:
            while True:
                (good, line) = Helper.get_multiline_from_file(infile)
                if not good:
                    break

                regex_match = self.regex_mips_plat.match(line)
                if not regex_match:
                    continue

                config = "CONFIG_" + regex_match.group(1)
                rhs = regex_match.group(2)
                fulldir = "arch/mips/" + rhs
                if os.path.isdir(fulldir):
                    current_precondition = DataStructures.Precondition()
                    current_precondition.add_condition(config)
                    local_arch_dirs[fulldir].append(current_precondition)

    def parse_mips_architecture(self, line, local_arch_dirs):
        """ Parses the main mips Makefile. Calls parse_mips_platform for
        any found Platform file in a subdirectory."""

        if self.arch != "mips":
            return False
        regex_match = self.regex_include.match(line)
        if not regex_match:
            return False

        with open(regex_match.group(1), "r") as included_file:
            for line in included_file:
                platform_match = self.regex_mips_platforms.match(line)
                if platform_match:
                    subpath = "arch/mips/" + platform_match.group(1) + \
                              "/Platform"
                    self.parse_mips_platform(subpath, local_arch_dirs)
        return True

    def parse_architecture_path(self, path, dirs_to_process):
        """ Gather additional information from an architecture
        directory. The arch-Makefiles behave a little bit differently
        depending on the architecture, which is why other helper
        routines are called for processing them."""

        if not os.path.isfile(path):
            logging.debug("arch/ parsing: no such file: " + path)
            return

        # Through the use of a defaultdict, we can directly call
        # local_arch_dirs[dir].append(..) without explicitely creating the
        # corresponding dict.
        local_arch_dirs = collections.defaultdict(list)
        basepath = os.path.dirname(path)

        logging.debug("Parsing architecture path " + basepath)

        with open(path, "r") as infile:
            while True:
                (good, line) = Helper.get_multiline_from_file(infile)
                if not good:
                    break

                line = line.replace(r"$(ARCH)", self.arch)
                line = line.replace(r"$(srctree)", ".")
                logging.debug("read line: " + line)
                if self.parse_architecture_regular(line, local_arch_dirs):
                    continue
                elif self.parse_arm_architecture(line, local_arch_dirs):
                    continue
                elif self.parse_blackfin_architecture(line, local_arch_dirs):
                    continue
                elif self.parse_mips_architecture(line, local_arch_dirs):
                    continue

        for item in local_arch_dirs:
            dirs_to_process[item] = \
                Helper.build_precondition(local_arch_dirs[item])

    def get_file_for_subdirectory(self, directory):
        """ Select the correct Kbuild makefile in directory."""
        if not directory.endswith('/'):
            directory += '/'
        descend = directory + "Kbuild"
        if not os.path.isfile(descend):
            descend = directory + "Makefile"
        return descend

    def process(self, parser, dirs_to_process, kernel_dir='.'):
        """ Here we can read the command line arguments, create global
        variables and insert items into a list of directories which will
        be processed. """

        parser.global_vars.create_variable("no_config_nesting", 0)

        # Default directories have no precondition
        for subdir in ["init/", "drivers/", "sound/", "firmware/", "net/",
                       "lib/", "usr/", "kernel/", "mm/", "fs/", "ipc/",
                       "security/", "crypto/", "block/", "certs/", "virt/"]:
            dirs_to_process[os.path.join(kernel_dir, subdir)] = DataStructures.Precondition()

        # Parse architecture specific path
        self.parse_architecture_path(os.path.join(kernel_dir, "arch", self.arch, "Makefile"),
                                     dirs_to_process)



class LinuxBefore(BaseClasses.BeforePass):
    """ Initialization of per-file variables for Linux."""

    def __init__(self, model, arch):
        """ Constructor for BeforeLinux, takes model and arch parameters"""
        super(LinuxBefore, self).__init__(model, arch)

    def process(self, parser, basepath):
        """ Initialize data structures before main processing loop."""
        # Composite object handling
        parser.local_vars.create_variable("composite_map",
            collections.defaultdict(DataStructures.Alternatives))

        # Main data structure
        parser.local_vars.create_variable("file_features",
            collections.defaultdict(DataStructures.Alternatives))

        # directory preconditions
        parser.local_vars.create_variable("dir_cond_collection",
            collections.defaultdict(DataStructures.Alternatives))

        # Updated with #if(n)def/#if(n)eq conditions
        parser.local_vars.create_variable("ifdef_condition",
            DataStructures.Precondition())

        # Local variable definitions
        parser.local_vars.create_variable("definitions", {})


class _00_LinuxDefinitions(BaseClasses.DuringPass):

    regex_subst_match = re.compile(r".*-\$\(subst m,y,\$\(" + CONFIG_FORMAT + r"\)\)\s*[\+|:|]=.*")
    regex_subst_sub = re.compile(r"(.*)-\$\(subst m,y,\$\(" + CONFIG_FORMAT + r"\)\)\s*(\+=|=|:=)(.*)")
    regex_my_match = re.compile(r".*-\$\(" + CONFIG_FORMAT + r":m=y\)\s*[\+|:|]=.*")
    regex_my_sub = re.compile(r"(.*)-\$\(" + CONFIG_FORMAT + r":m=y\)\s*(\+=|=|:=)(.*)")
    regex_def = re.compile(r"\s*([A-Z_-]+)\s*[\?:]?=\s*(.*)")

    def __init__(self, model, arch):
        super(_00_LinuxDefinitions, self).__init__(model, arch)

    def do_line_replacements(self, parser, line):
        """ Replace occurences of known patterns and definitions in line and
        return them. """

        # "obj-$(subst m,y,$(CONFIG_XY)) ?? obj.o" becomes "obj-$(CONFIG_XY) ?? obj.o"
        if self.regex_subst_match.match(line):
            line = self.regex_subst_sub.sub(r"\1-$(CONFIG_\2) \3 \4", line)
        # "obj-$(CONFIG_XY:m=y) ?? obj.o" becomes "obj-$(CONFIG_XY) ?? obj.o"
        if self.regex_my_match.match(line):
            line = self.regex_my_sub.sub(r"\1-$(CONFIG_\2) \3 \4", line)

        # Replacement of defined macros
        for definition in parser.local_vars["definitions"]:
            if r"$(" + definition + r")" in line:
                line = re.sub(r"\$\(" + definition + r"\)",
                              parser.local_vars["definitions"][definition],
                              line)

        return line

    def process(self, parser, line, basepath):
        _line = line.raw_line
        # Assumes definitions are UPPERCASE and done with :=
        definition = self.regex_def.match(_line)
        if definition:
            # Recursive definitions are ugly for now, so ignore them.
            if definition.group(1) in definition.group(2):
                return False

            parser.local_vars["definitions"][definition.group(1)] = \
                definition.group(2)
            return True
        else:
            line.processed_line = self.do_line_replacements(parser, _line)
            return False


class _01_LinuxIf(BaseClasses.DuringPass):
    """ Evaluation of ifdef/ifeq conditions in Linux."""

    def __init__(self, model, arch):
        super(_01_LinuxIf, self).__init__(model, arch)

    def process(self, parser, line, basepath):
        """ Process lines starting with if{n}{eq,def}. If the condition could
        not be properly parsed, the line and all following lines until the
        corresponding endif are marked as invalid - update_if_condition will
        return False in that case, as the nesting level has been incremented to
        a value > 0. """
        _line = line.processed_line
        retval = update_if_condition(_line, parser.local_vars["ifdef_condition"],
                                     parser.global_vars, parser.local_vars,
                                     self.model)
        line.condition = parser.local_vars["ifdef_condition"][:]
        line.invalid = retval
        return retval


class _02_LinuxObjects(BaseClasses.DuringPass):
    """ Evaluation of lines describing object files in Linux."""

    obj_line = r"\s*(obj|lib)-(y|m|\$[\(\{]" + CONFIG_FORMAT + \
               r"[\)\}])\s*(:=|\+=|=)\s*(([A-Za-z0-9.,_\$\(\)/-]+\s*)+)"
    regex_obj = re.compile(obj_line)
    subdir_line = r"\s*subdir-(y|\$\(" + CONFIG_FORMAT + r"\))\s*\+=(.*)"
    regex_subdir = re.compile(subdir_line)

    def __init__(self, model, arch):
        super(_02_LinuxObjects, self).__init__(model, arch)

    def __process(self, parser, line, basepath):
        line = line.processed_line

        # Try obj-... +=/:=/= objfile.o
        regex_match = self.regex_obj.match(line)
        if regex_match:
            rhs = regex_match.group(5)
            # Fixes one special case in arch/mips/lib, could be extended with
            # other make-commands.
            if rhs.startswith("$(filter-out"):
                return False
            # First, check for match on obj-y or obj-m
            if regex_match.group(2) == "y" or regex_match.group(2) == "m":
                matches = [x for x in re.split("\t| ", rhs) if x]
                for match in matches:
                    fullpath = basepath + "/" + match
                    if os.path.isdir(fullpath):
                        parser.local_vars["dir_cond_collection"][fullpath].\
                            add_alternative(
                                parser.local_vars["ifdef_condition"][:]
                            )
                    else:
                        sourcefile = Helper.guess_source_for_target(fullpath)
                        if sourcefile:
                            parser.local_vars["file_features"][sourcefile].\
                                add_alternative(
                                    parser.local_vars["ifdef_condition"][:]
                                )
                        else:
                            parser.local_vars["composite_map"][fullpath].\
                                add_alternative(
                                    parser.local_vars["ifdef_condition"][:]
                                )
            else:
                # Then test obj-$(CONFIG_XY)
                config = regex_match.group(3)

                condition = Helper.get_config_string(config, self.model)

                matches = [x for x in re.split("\t| ", rhs) if x]

                parser.local_vars["ifdef_condition"].add_condition(condition)

                for match in matches:
                    fullpath = basepath + "/" + match
                    if os.path.isdir(fullpath):
                        # Has this directory been picked up via subdir-XY?
                        if parser.local_vars["ifdef_condition"] in \
                                parser.local_vars["dir_cond_collection"][fullpath]:
                            continue
                        parser.local_vars["dir_cond_collection"][fullpath].\
                            add_alternative(
                                parser.local_vars["ifdef_condition"][:]
                            )
                    else:
                        sourcefile = Helper.guess_source_for_target(fullpath)
                        if sourcefile:
                            parser.local_vars["file_features"][sourcefile].\
                                add_alternative(
                                    parser.local_vars["ifdef_condition"][:]
                                )
                        # Remove 'else' if composite lists with the name of
                        # a source file exist (drivers/media/pci/mantis/)
                        else:
                            parser.local_vars["composite_map"][fullpath].\
                                add_alternative(
                                    parser.local_vars["ifdef_condition"][:]
                                )

                parser.local_vars["ifdef_condition"].pop()
        # Up to v3.15, the security/ subdir used only subdir-$(CONFIG..) for
        # descending into subdirectories. All other directories didn't...
        else:
            match = self.regex_subdir.match(line)
            if not match:
                return False

            # If subdir is conditional, add condition to ifdef_condition
            if match.group(1) != "y":
                condition = Helper.get_config_string(match.group(2), self.model)
                parser.local_vars["ifdef_condition"].add_condition(condition)

            rhs = match.group(3)
            rhs_matches = [x.rstrip("/") for x in re.split("\t| ", rhs) if x]
            for m in rhs_matches:
                fullpath = basepath + "/" + m + "/"
                if not os.path.isdir(fullpath):
                    continue

                # Has this directory been picked up via obj-XY?
                if parser.local_vars["ifdef_condition"] in \
                        parser.local_vars["dir_cond_collection"][fullpath]:
                    continue

                parser.local_vars["dir_cond_collection"][fullpath].\
                    add_alternative(
                        parser.local_vars["ifdef_condition"][:]
                    )

            if match.group(1) != "y":
                parser.local_vars["ifdef_condition"].pop()

        return True

    def process(self, parser, line, basepath):
        if "$(BITS)" in line.processed_line:
            line_1 = DataStructures.LineObject(line.raw_line)
            line_1.processed_line = line.processed_line.replace(r"$(BITS)", "32")
            line_1.condition = line.condition
            line_1.invalid = line.invalid
            line_2 = DataStructures.LineObject(line.raw_line)
            line_2.processed_line = line.processed_line.replace(r"$(BITS)", "64")
            line_2.condition = line.condition
            line_2.invalid = line.invalid

            lines = [line_1, line_2]
        else:
            lines = [line]

        for line in lines:
            self.__process(parser, line, basepath)


class _01_LinuxExpandMacros(BaseClasses.AfterPass):
    """ Expand macros in Linux Makefiles."""

    # Static strings/regexes
    regex_base = re.compile(r"([A-Za-z0-9,_-]+)\.o|\$\(([A-Za-z0-9,_-]+)\)")

    def __init__(self, model, arch):
        """ Constructor for _01_LinuxExpandMacros. """
        super(_01_LinuxExpandMacros, self).__init__(model, arch)

    def expand_macro(self, name, path, condition, already_expanded, parser, maxdepth=3):
        """ Expand a macro named @name. Preconditions to the folder are given
        in @condition. The input file is @path and to avoid endless
        recursion processing is aborted if the current name is already present
        in @already_expanded. To save the results, the local variables are
        accessed via the @parser parameter."""

        if maxdepth == 0:
            return

        if name in already_expanded:
            return
        else:
            already_expanded.add(name)

        basepath = os.path.dirname(name)
        filename = os.path.basename(name)

        basename = ""

        # Extract base name from macro name
        match = self.regex_base.match(filename)
        if not match:
            return

        if match.group(1) is None:  # we have a macro
            basename = match.group(2)
            if basename.endswith("y"):
                basename = basename[:-1]
        elif match.group(2) is None:  # we have a file
            basename = match.group(1)

        scan_regex_string = ""
        if match.group(1) is None:
            scan_regex_string = r"\s*" + basename + r"(|y|\$\(" + \
                                CONFIG_FORMAT + r"\))\s*(:=|\+=|=)\s*(.*)"
        else:
            scan_regex_string = r"\s*" + basename + r"(|-y|-objs|-\$\(" + \
                                CONFIG_FORMAT + r"\))\s*(:=|\+=|=)\s*(.*)"

        scan_regex = re.compile(scan_regex_string)

        if not path in parser.file_content_cache:
            parser.read_whole_file(path)

        inputs = parser.file_content_cache[path]

        for line in inputs:
            if line.invalid:
                continue

            ifdef_condition = line.condition
            line = line.processed_line

            match = scan_regex.match(line)
            if not match:
                continue

            config_in_composite = match.group(2)
            condition_comp = ""
            if config_in_composite:
                condition_comp = Helper.get_config_string(config_in_composite,
                                                         self.model)

            rhs = match.group(4)

            matches = [x for x in re.split("\t| ", rhs) if x]
            for item in matches:
                fullpath = basepath + "/" + item
                passdown_condition = condition[:]
                if config_in_composite:
                    passdown_condition.append(condition_comp)

                if os.path.isdir(fullpath):
                    parser.local_vars["dir_cond_collection"]\
                        [fullpath].add_alternative(passdown_condition[:])
                else:
                    sourcefile = Helper.guess_source_for_target(fullpath)
                    if not sourcefile:
                        self.expand_macro(fullpath, path,passdown_condition,
                                          already_expanded, parser, maxdepth-1)
                    else:
                        full_condition = DataStructures.Precondition()
                        if len(condition) > 0:
                            full_condition = condition[:]
                        if config_in_composite:
                            full_condition.append(condition_comp)
                        if len(ifdef_condition) > 0:
                            full_condition.extend(ifdef_condition)

                        parser.local_vars["file_features"][sourcefile].\
                            add_alternative(full_condition[:])

        already_expanded.discard(name)

    def process(self, parser, path, condition_for_current_dir):
        """ Process macros from composite_map variable. """
        # Macro expansion
        for obj in parser.local_vars.get_variable("composite_map"):
            downward_condition = Helper.build_precondition(parser.\
                                    local_vars["composite_map"][obj])
            already_expanded = set()

            # Pass an empty set as the already_expanded parameter, as
            # expand_macro is allowed to call itself recursively
            self.expand_macro(obj, path, downward_condition,
                              already_expanded, parser)


class _02_LinuxProcessSubdirectories(BaseClasses.AfterPass):
    """ Process subdirectories in Linux."""

    def __init__(self, model, arch):
        """ Constructor for _02_LinuxProcessSubdirectories. """
        super(_02_LinuxProcessSubdirectories, self).__init__(model, arch)

    def process(self, parser, path, condition_for_current_dir):
        """ Process all subdirectories."""
        for directory in parser.local_vars["dir_cond_collection"]:
            downward_condition = Helper.build_precondition(
                parser.local_vars["dir_cond_collection"][directory],
                condition_for_current_dir)
            descend = parser.init_class.get_file_for_subdirectory(directory)

            parser.process_kbuild_or_makefile(descend, downward_condition)


class _03_LinuxOutput(BaseClasses.AfterPass):
    """ Output class for Linux."""

    def __init__(self, model, arch):
        """ Constructor for _03_LinuxOutput. """
        super(_03_LinuxOutput, self).__init__(model, arch)
        self.config = {}

    def process(self, parser, path, condition_for_current_dir):
        """ Print conditions collected in file_features variable. """
        for item in sorted(parser.local_vars["file_features"]):
            # Build the precondition, including the conditions for the current
            # directory.
            precondition = Helper.\
                    build_precondition(parser.local_vars["file_features"][item],
                                       condition_for_current_dir)

            full_string = " & ".join(precondition)
            self.config[item] = full_string



#!/usr/bin/env python3

import pytest
import os

from cvehound.exception import UnsupportedVersion

def test_on_fix(hound, repo, cve):
    fix = hound.get_rule_fix(cve)

    repo.git.checkout('--force', fix)
    try:
        assert not hound.check_cve(cve), cve + ' fails on fix commit'
        repo.git.checkout(fix + '~')
        assert hound.check_cve(cve), cve + ' fails to detect fix~ commit'
    except UnsupportedVersion:
        pytest.skip('Unsupported spatch version')



#!/usr/bin/env python3

import os
import sys
import pkg_resources
import shutil
from urllib.request import urlopen
from io import BytesIO
from zipfile import ZipFile

def main(args=sys.argv):
    rule_dir = pkg_resources.resource_filename('cvehound', 'cve')

    for rule in os.listdir(rule_dir):
        entry = os.path.join(rule_dir, rule)
        try:
            if os.path.isdir(entry):
                shutil.rmtree(entry)
            else:
                os.unlink(entry)
        except Exception as e:
            print('Failed to delete {}: {}'.format(entry, e))

    with urlopen('https://github.com/evdenis/cvehound/archive/refs/heads/master.zip') as uh:
        rules = []
        with ZipFile(BytesIO(uh.read())) as zh:
            rules = filter(lambda x: x.startswith('cvehound-master/cvehound/cve'), zh.namelist())
            rules = list(rules)
            zh.extractall(path=rule_dir, members=rules)
        prefix_len = len('cvehound-master/cvehound/cve') + 1
        os.chdir(rule_dir)
        for entry in rules:
            to = entry[prefix_len:].strip()
            if not to or not (os.path.isfile(entry) or os.path.isdir(entry)):
                continue
            os.rename(entry, to)
        shutil.rmtree('cvehound-master')

if __name__ == '__main__':
    main(sys.argv)



#!/usr/bin/env python3

import pytest
import os
import re
from git import GitCommandError

from cvehound.exception import UnsupportedVersion

@pytest.mark.slow
def test_between_fixes_fix(hound, repo, cve):
    fix = hound.get_rule_fix(cve)
    fixes = hound.get_rule_fixes(cve)
    files = hound.get_rule_files(cve)
    pathspec = re.compile(r"error: pathspec '([^']+)'")

    repo.git.checkout('--force', fixes)

    commits = repo.git.log('--format=%H', '--no-merges', '--ancestry-path', fixes + '..' + fix + '~', '--', files)
    for commit in commits.split():
        checkout_files = files
        try:
            repo.git.checkout('--force', commit, '--', checkout_files)
        except GitCommandError as e:
            remove_files = set(pathspec.findall(e.stderr))
            repo.git.checkout('--force', commit, '--', list(set(checkout_files) - remove_files))

        try:
            assert hound.check_cve(cve), cve + ' fails to detect on ' + commit
        except UnsupportedVersion:
            pytest.skip('Unsupported spatch version')



CWE = {
    "Access of Resource Using Incompatible Type ('Type Confusion')" : [843],
    "Access of Uninitialized Pointer": [824],
    "Allocation of Resources Without Limits or Throttling": [770],
    "Authentication Errors": [1211, 288, 290, 294, 295, 296, 299, 303, 304, 305, 306, 307, 308, 309, 322, 603, 645, 804, 846],
    "Authentication Issues": [1211, 288, 290, 294, 295, 296, 299, 303, 304, 305, 306, 307, 308, 309, 322, 603, 645, 804, 846],
    "Buffer Access with Incorrect Length Value": [805],
    "Buffer Copy without Checking Size of Input ('Classic Buffer Overflow')": [120],
    "Buffer Errors": [1218, 120, 123, 124, 125, 131, 786, 787, 788, 805],
    "Cleartext Transmission of Sensitive Information": [319],
    "Code Injection": [94, 96],
    "Code": [94, 96, 393, 394, 215, 540, 470, 1265],
    "Command Injection": [77, 78],
    "Concurrent Execution using Shared Resource with Improper Synchronization ('Race Condition')": [362],
    "Configuration": [16],
    "Credentials Management": [255, 256, 257, 260, 261, 262, 263, 324, 521, 523, 549, 620, 640, 798, 916],
    "Cryptographic Issues": [310, 261, 324, 325, 328, 331, 334, 335, 338, 347, 916, 1240],
    "DEPRECATED: Information Exposure Through Debug Log Files": [534],
    "Data Handling": [19, 130, 166, 167, 168, 178, 182, 186, 229, 233, 237, 241, 409, 471, 472, 601, 611, 624, 625, 776, 1024],
    "Data Processing Errors": [19, 130, 166, 167, 168, 178, 182, 186, 229, 233, 237, 241, 409, 471, 472, 601, 611, 624, 625, 776, 1024],
    "Deadlock": [833],
    "Divide By Zero": [369],
    "Double Free": [415],
    "Error Handling": [728, 7, 203, 209, 228, 252, 289, 390, 391, 394, 636],
    "Excessive Iteration": [834],
    "Expected Behavior Violation": [440],
    "Exposure of Resource to Wrong Sphere": [668],
    "Exposure of Sensitive Information to an Unauthorized Actor": [200],
    "Format String Vulnerability": [133, 134, 135, 597],
    "Heap-based Buffer Overflow": [122],
    "Improper Access Control": [284],
    "Improper Authentication": [287],
    "Improper Authorization": [285],
    "Improper Check for Unusual or Exceptional Conditions": [754],
    "Improper Control of Dynamically-Managed Code Resources": [913],
    "Improper Control of Generation of Code ('Code Injection')": [94],
    "Improper Error Handling": [728, 7, 203, 209, 228, 252, 289, 390, 391, 394, 636],
    "Improper Handling of Exceptional Conditions": [755],
    "Improper Initialization": [665],
    "Improper Input Validation": [20],
    "Improper Limitation of a Pathname to a Restricted Directory ('Path Traversal')": [22],
    "Improper Locking": [667],
    "Improper Neutralization of Special Elements in Output Used by a Downstream Component ('Injection')": [74],
    "Improper Neutralization of Special Elements used in a Command ('Command Injection')": [77],
    "Improper Preservation of Permissions": [281],
    "Improper Privilege Management": [269],
    "Improper Resource Shutdown or Release": [404],
    "Improper Restriction of Communication Channel to Intended Endpoints": [923],
    "Improper Restriction of Excessive Authentication Attempts": [307],
    "Improper Restriction of Operations within the Bounds of a Memory Buffer": [119],
    "Improper Validation of Array Index": [129],
    "Improper Validation of Integrity Check Value": [354],
    "Improper Verification of Cryptographic Signature": [347],
    "Improperly Implemented Security Check for Standard": [358],
    "Inadequate Encryption Strength": [326],
    "Incomplete Cleanup": [459],
    "Incorrect Authorization": [863],
    "Incorrect Calculation of Buffer Size": [131],
    "Incorrect Calculation": [682],
    "Incorrect Comparison": [697],
    "Incorrect Conversion between Numeric Types": [681],
    "Incorrect Default Permissions": [276],
    "Incorrect Permission Assignment for Critical Resource": [732],
    "Incorrect Type Conversion or Cast": [704],
    "Information Exposure Through Log Files": [532],
    "Information Exposure": [200, 201, 203, 209, 213, 215, 359, 497, 538, 1243, 1258, 1273, 1295],
    "Information Leak / Disclosure": [200, 201, 203, 209, 213, 215, 359, 497, 538, 1243, 1258, 1273, 1295],
    "Injection": [1026, 1027, 77, 78, 88, 917, 943],
    "Input Validation": [1019, 20, 59, 74, 75, 76, 77, 78, 88, 94, 96, 99, 138, 349, 502, 641, 790, 791, 795],
    "Insecure Default Initialization of Resource": [1188],
    "Insertion of Sensitive Information into Externally-Accessible File or Directory": [538],
    "Insufficient Information": [778, 223, 221, 222],
    "Insufficient Verification of Data Authenticity": [345],
    "Integer Overflow or Wraparound": [190],
    "Integer Underflow (Wrap or Wraparound)": [191],
    "Interpretation Conflict": [436],
    "Link Following": [59, 61, 62],
    "Loop with Unreachable Exit Condition ('Infinite Loop')": [835],
    "Memory Buffer Errors": [1218, 120, 123, 124, 125, 131, 786, 787, 788, 805],
    "Missing Authentication for Critical Function": [306],
    "Missing Authorization": [862, 638, 1314],
    "Missing Initialization of Resource": [909],
    "Missing Release of Memory after Effective Lifetime": [401],
    "Missing Release of Resource after Effective Lifetime": [772],
    "NULL Pointer Dereference": [476],
    "Non-exit on Failed Initialization": [455],
    "Numeric Errors": [189, 128, 190, 191, 192, 193, 197, 198, 369, 681, 839, 1077],
    "Observable Differences in Behavior to Error Inputs": [203, 205],
    "Observable Discrepancy": [203],
    "Off-by-one Error": [193],
    "Operation on a Resource after Expiration or Release": [672],
    "Other": [],
    "Out-of-bounds Read": [125],
    "Out-of-bounds Write": [787],
    "Path Traversal": [22, 23, 36],
    "Permissions, Privileges, and Access Control": [264],
    "Permissions, Privileges, and Access Controls": [264],
    "Privilege Dropping / Lowering Errors": [271],
    "Race Conditions": [362, 364, 366, 367, 368, 421, 689, 1223, 1298],
    "Release of Invalid Pointer or Reference": [763],
    "Resource Management Errors": [399, 73, 403, 410, 470, 502, 619, 641, 694, 763, 770, 771, 772, 826, 908, 909, 910, 911, 914, 915, 920, 1188],
    "Security Features": [254, 256, 258, 259, 260, 261, 272, 284, 285, 330, 359, 798],
    "Stack-based Buffer Overflow": [121],
    "String Errors": [133, 134, 135, 597],
    "Time-of-check Time-of-use (TOCTOU) Race Condition": [367],
    "Unchecked Return Value": [252],
    "Uncontrolled Resource Consumption ('Resource Exhaustion')": [400],
    "Uncontrolled Resource Consumption": [400],
    "Unspecified": [],
    "Use After Free": [416],
    "Use of Insufficiently Random Values": [330],
    "Use of Uninitialized Resource": [908],
    "Use of a Broken or Risky Cryptographic Algorithm": [327],
}



#!/usr/bin/env python3

import pytest
import os

from cvehound.exception import UnsupportedVersion

@pytest.mark.slow
def test_on_fixes(hound, repo, cve):
    fixes = hound.get_rule_fixes(cve)

    repo.git.checkout('--force', fixes)
    try:
        assert hound.check_cve(cve), 'fails to detect on fixes tag'

        if fixes != 'v2.6.12-rc2' and \
           fixes != '1da177e4c3f41524e886b7f1b8a0c1fc7321cac2':
            repo.git.checkout('HEAD~')
            assert not hound.check_cve(cve), 'detects on fixes~ tag'
    except UnsupportedVersion:
        pytest.skip('Unsupported spatch version')



#!/usr/bin/env python3

import os
import sys
import subprocess
import logging
from subprocess import PIPE
import collections
from datetime import datetime
from sympy.logic import simplify_logic
from cvehound.exception import UnsupportedVersion
from cvehound.util import get_spatch_version, get_rule_cves, get_cves_metadata, parse_coccinelle_output
from cvehound.kbuild import KbuildParser
from cvehound.config import Config

__VERSION__ = '1.2.1'


class CVEhound:

    def __init__(self, kernel, metadata=None, config=None, check_strict=False, arch='x86'):
        kernel = os.path.abspath(kernel)
        self.kernel = kernel
        self.metadata = get_cves_metadata(metadata)
        self.spatch_version = get_spatch_version()
        self.check_strict = check_strict
        self.rules_metadata = {}
        (self.cve_all_rules, self.cve_assigned_rules, self.cve_disputed_rules) = get_rule_cves()

        ipaths = [
            os.path.join('arch', arch, 'include'),
            os.path.join('arch', arch, 'include/generated'),
            os.path.join('arch', arch, 'include/uapi'),
            os.path.join('arch', arch, 'include/generated/uapi'),
            'include',
            'include/uapi',
            'include/generated/uapi'
        ]
        ipaths = map(lambda f: os.path.join(kernel, f), ipaths)
        includes = []
        for i in ipaths:
            includes.append('-I')
            includes.append(i)
        self.includes = includes

        if config:
            parser = KbuildParser(None, arch)
            dirs_to_process = collections.OrderedDict()
            parser.init_class.process(parser, dirs_to_process, kernel)

            for item in dirs_to_process:
                descend = parser.init_class.get_file_for_subdirectory(item)
                parser.process_kbuild_or_makefile(descend, dirs_to_process[item])

            self.config_map = parser.get_config()
            if config != '-':
                self.config_file = config
                self.config = Config(config)
            else:
                self.config_file = None
                self.config = None
        else:
            self.config_file = None
            self.config_map = None
            self.config = None

        if self.spatch_version <= 104:
            logging.warning('spatch (coccinelle) version is too old.\n'
                            'Please, consider updating to >= 1.0.7 version.')

    def get_grep_pattern(self, rule):
        patterns = []
        with open(rule, 'rt') as fh:
            for line in fh:
                line = line.strip()
                if not line:
                    continue
                if line.startswith('//'):
                    continue
                if line:
                    patterns.append(line)
        return patterns

    def _print_found_cve(self, cve):
        logging.warning('Found: ' + cve)
        if cve in self.metadata:
            info = self.metadata[cve]
            if 'cmt_msg' in info:
                logging.info('MSG: ' + info['cmt_msg'])
            if 'cwe' in info:
                logging.info('CWE: ' + info['cwe'])
            if 'cvss2' in info and 'score' in info['cvss2']:
                logging.info('CVSS2: ' + str(info['cvss2']['score']))
            if 'cvss3' in info and 'score' in info['cvss3']:
                logging.info('CVSS3: ' + str(info['cvss3']['score']))
            if 'fix_date' in info:
                logging.info('FIX DATE: ' + str(datetime.utcfromtimestamp(info['fix_date']).strftime('%Y-%m-%d')))
        logging.info('https://www.linuxkernelcves.com/cves/' + cve)

    def _print_affected_files(self, config):
        if 'files' in config and config['files']:
            logging.info('Affected Files:')
            for file in config['files']:
                logic = config['files'][file]['logic']
                if self.config:
                    affected = 'affected' if config['files'][file]['config'] else 'not affected'
                    logging.info(' - ' + file + ': ' + logic + '\n   ' + self.config_file + ': ' + affected)
                else:
                    logging.info(' - ' + file + ': ' + logic)

        if 'affected' not in config or config['affected'] == None:
            return
        config_affected = 'affected' if config['affected'] else 'not affected'
        if self.config:
            logging.info('Config: ' + self.config_file + ' ' + config_affected)
        else:
            logging.info('Config: any ' + config_affected)

    def check_cve(self, cve, all_files=False):
        result = {}
        is_grep = False
        rule = self.cve_all_rules[cve]
        if rule.endswith('.grep'):
            is_grep = True

        files = []
        if not all_files:
            files = self.get_rule_files(cve)
            files = map(lambda f: os.path.join(self.kernel, f), files)
            files = filter(lambda f: os.path.exists(f), files)
            files = list(files)
        if not files:
            files = [ self.kernel ]

        includes = self.includes.copy()
        kconfig = os.path.join(self.kernel, 'include/linux/kconfig.h')
        if os.path.exists(kconfig):
            includes.append('--include')
            includes.append(kconfig)

        logging.debug('Checking: ' + cve)

        output = ''
        run = None
        if not is_grep:
            rule_ver = self.get_rule_version(cve)
            if rule_ver and rule_ver > self.spatch_version:
                raise UnsupportedVersion(self.spatch_version, cve, rule_ver)
            try:
                cocci_cmd = ['spatch', '--no-includes', '--include-headers',
                             '-D', 'detect', '--chunksize', '1', '-j', '1',
                             '--no-show-diff', '--very-quiet',
                             *includes]
                if self.spatch_version > 104: # Not suppored on coccinelle 1.0.4
                    cocci_cmd.extend(['--python', os.path.realpath(sys.executable)])
                cocci_cmd.extend(['--cocci-file', rule, *files])

                logging.debug(' '.join(cocci_cmd))

                run = subprocess.run(cocci_cmd, stdout=PIPE, stderr=PIPE, check=True, universal_newlines=True)
                output = run.stdout.strip()
            except subprocess.CalledProcessError as e:
                err = e.stderr.split('\n')[-2]
                # Coccinelle 1.0.4 bug workaround
                if ('Sys_error("' + cve + ': No such file or directory")') not in err:
                    raise e
        else:
            for pattern in self.get_grep_pattern(rule):
                args = ['grep', '-rPzle', pattern, *files]
                run = subprocess.run(args, stdout=PIPE, stderr=PIPE, check=False, universal_newlines=True)
                if run.returncode != 0:
                    break
                output += run.stdout.strip()
            else:
                # Found all patterns
                output += '\nERROR'

        if 'ERROR' not in output:
            return False

        config_result = {}
        if self.config_map:
            kernel_files = {}
            for line in output.split('\n'):
                file = []
                if not is_grep:
                    file = [ line.split(':')[0] ]
                else:
                    while True:
                        try:
                            rindex = line.rindex(self.kernel)
                        except ValueError:
                            break
                        file.append(line[rindex:])
                        line = line[:rindex]
                for f in filter(lambda f: os.path.isfile(f), file):
                    kernel_files[f] = self.config_map.get(f, '')
            if kernel_files:
                config_affected = None
                if 'files' not in config_result:
                    config_result['files'] = {}
                for file, config in kernel_files.items():
                    rel_file = file[len(self.kernel)+1:]
                    result_file = {}
                    if config:
                        config = simplify_logic(config)
                        result_file['logic'] = str(config)
                        if self.config:
                            affected = config.subs(self.config.get_mapping())
                            if affected == True:
                                result_file['config'] = True
                                config_affected = True
                            else:
                                result_file['config'] = False
                                if config_affected == None:
                                    config_affected = False
                    elif not file.endswith('.h'):
                        result_file['logic'] = str(True)
                        result_file['config'] = True
                        config_affected = True
                    elif file.endswith('.h'): # FIXME: only .h file?
                        result_file['logic'] = str(True)
                        result_file['config'] = True
                        config_affected = True
                    config_result['files'][rel_file] = result_file
            if config_affected != None:
                config_result['affected'] = config_affected

        if self.check_strict and 'affected' in config_result and config_result['affected'] or not self.check_strict:
            if cve in self.metadata:
                result = self.metadata[cve]
            result['config'] = config_result
            result['spatch_output'] = output
            if not is_grep:
                result['files'] = parse_coccinelle_output(output)
            else:
                result['files'] = list(map(lambda x: { "file": x } , files))
            self._print_found_cve(cve)
            self._print_affected_files(config_result)
            logging.debug(output)
            logging.info('')
        else:
            return False

        return result

    def get_rule_metadata(self, cve):
        files = []
        fix = None
        fixes = None
        version = 0

        if cve in self.rules_metadata:
            return self.rules_metadata[cve]

        with open(self.cve_all_rules[cve], 'rt') as fh:
            for line in fh:
                if not line.startswith('///'):
                    break
                if 'Files:' in line:
                    files = line.partition('Files:')[2].split()
                elif 'Fix:' in line:
                    fix = line.partition('Fix:')[2].strip()
                elif 'Fixes:' in line:
                    fixes = line.partition('Fixes:')[2].strip()
                elif 'Detect-To:' in line:
                    fixes = line.partition('Detect-To:')[2].strip()
                elif 'Version:' in line:
                    version = line.partition('Version:')[2].strip()
                    version = int(version.replace('.', ''))

        meta = {
            'files': files,
            'fix': fix,
            'fixes': fixes,
            'version': version
        }
        self.rules_metadata[cve] = meta
        return meta

    def get_cve_metadata(self, cve):
        return self.metadata.get(cve, {})

    def get_cve_cwe(self, cve):
        return self.get_cve_metadata(cve).get('cwe', None)

    def get_cve_exploit(self, cve):
        return self.get_cve_metadata(cve).get('exploit', False)

    def get_all_cves(self):
        return set(self.cve_all_rules.keys())

    def get_assigned_cves(self):
        return set(self.cve_assigned_rules.keys())

    def get_disputed_cves(self):
        return set(self.cve_disputed_rules.keys())

    def get_rule(self, cve):
        return self.cve_all_rules[cve]

    def get_rule_fix(self, cve):
        return self.get_rule_metadata(cve)['fix']

    def get_rule_fixes(self, cve):
        return self.get_rule_metadata(cve)['fixes']

    def get_rule_files(self, cve):
        return self.get_rule_metadata(cve)['files']

    def get_rule_version(self, cve):
        return self.get_rule_metadata(cve)['version']




class UnsupportedVersion(Exception):
    def __init__(self, spatch_version, cve, rule_version):
        self.spatch_version = '.'.join(str(spatch_version))
        self.cve = cve
        self.rule_version = '.'.join(str(rule_version))




""" Data structures useful for kbuildparse modules."""

# Copyright (C) 2014-2015 Andreas Ruprecht <andreas.ruprecht@fau.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

class VariableStore(dict):
    """ Class managing variables for easy access across different
    subclasses."""

    def __init__(self):
        super(VariableStore, self).__init__()
        self.nxt = None

    def create_variable(self, name, variable):
        """ Create a variable with name referenced in @name."""
        self[name] = variable

    def get_variable(self, name):
        """ Get a variable of name @name from the store. Returns None,
        if no corresponding variable was found."""
        if name in self:
            return self[name]
        return None

    def increment_variable(self, name, amount=1):
        """ int has no reference semantics, so incrementing a number
        requires rewriting the entry in the dictionary."""
        self[name] += amount

    def decrement_variable(self, name, amount=1):
        """ see increment_variable """
        self[name] -= amount


class Precondition(list):
    """ Class representing a list of preconditions for a file."""

    def add_condition(self, condition, keep_duplicates=True):
        """ Add a condition to this Precondition."""
        if keep_duplicates or condition not in self:
            self.append(condition)

    def __hash__(self):
        """ Hashing is deferred to superclass."""
        return hash(super(Precondition, self))


class Alternatives(list):
    """ Class representing a list of alternative Preconditions."""

    def add_alternative(self, precondition):
        """ Add an alternative Precondition."""
        self.append(precondition)


class LineObject(object):
    """ Class representing a line with conditions."""

    def __init__(self, line):
        self.raw_line = line
        self.processed_line = line
        self.condition = []
        # Invalid means that this line is inside a if{n}{def,eq} block for which
        # we could not evaluate the correct condition.
        self.invalid = False



import os
import re
import subprocess
import pkg_resources
import json
import gzip
from shutil import which
from configparser import ConfigParser

def tool_exists(name):
    return which(name) is not None

def removesuffix(string, suffix):
    if suffix and string.endswith(suffix):
        return string[:-len(suffix)]
    return string[:]

def get_config_data(path):
    with open(path, 'rt') as fh:
        ver_pattern = re.compile(r'# Linux/(\S+)\s+(\S+)\s+Kernel Configuration')
        for line in fh:
            res = ver_pattern.match(line)
            if res:
                return {'arch': res.group(1), 'version': res.group(2)}
    return {}

def get_kernel_version(path):
    with open(os.path.join(path, 'Makefile'), 'rt') as fh:
        makefile = fh.read()
    version = {}
    for key in ['version', 'patchlevel', 'sublevel', 'extraversion', 'name']:
        res = re.search('^' + key.upper() + r'[ \t]*=[ \t]*(.*)[ \t]*$', makefile, re.MULTILINE)
        if res:
            version[key] = res.group(1)
        else:
            version[key] = ''
    version['full'] = '.'.join([version['version'], version['patchlevel'], version['sublevel']]) + version['extraversion']
    return version

def get_cvehound_version():
    version = pkg_resources.get_distribution('cvehound').version
    location = pkg_resources.get_distribution('cvehound').location

    if not os.path.exists(os.path.join(location, '.git')):
        return version

    try:
        desc = ['git', 'describe', '--tags', '--dirty']
        version = subprocess.check_output(
            desc, cwd=location, stderr=subprocess.DEVNULL, universal_newlines=True
        ).strip()
    finally:
        return version

def get_spatch_version():
    version = subprocess.check_output(
            ['spatch', '--version'],
            stderr=subprocess.DEVNULL, universal_newlines=True
    ).strip().split('\n')[0]
    res = re.match(r'spatch\s+version\s+([\d.]+)', version)
    version_string = res.group(1)
    nums = version_string.count('.')
    if nums == 1:
        version_string += '.0'
    return int(version_string.replace('.', ''))

def get_rule_cves():
    known = {}
    assigned = {}
    disputed = {}
    for root, dirs, files in os.walk(pkg_resources.resource_filename('cvehound', 'cve/')):
        for cve in files:
            path = os.path.join(root, cve)
            name = removesuffix(removesuffix(cve, '.grep'), '.cocci')
            known[name] = path
            if 'disputed' in root:
                disputed[name] = path
            else:
                assigned[name] = path
    return (known, assigned, disputed)

def get_cves_metadata(path):
    if not path:
        path = os.environ.get('CVEHOUND_METADATA',
               pkg_resources.resource_filename('cvehound', 'data/kernel_cves.json.gz'))
    data = None
    with gzip.open(path, 'rt', encoding='utf-8') as fh:
        data = json.load(fh)
    return data

def parse_coccinelle_output(output):
    files = []
    for line in output.splitlines():
        file, hline, _ = line.split(':', 2)
        files.append({
            'file': file,
            'line': int(hline),
        })
    return files

def parse_config(file):
    parser = ConfigParser()
    with open(file, 'rt') as fh:
        parser.read_string("[cvehound]\n" + fh.read())
    config = dict(parser['cvehound'])

    for key in ['cve', 'exclude', 'cwe', 'files', 'ignore_files']:
        if key not in config:
            continue
        config[key] = config[key].split()

    if 'verbose' in config:
        try:
            config['verbose'] = int(config['verbose'])
        except ValueError:
            raise Exception('"verbose" should be an integer')

    for key in ['check_strict', 'all_files', 'exploit']:
        if key not in config:
            continue
        if config[key].lower() in ['y', 't', '1', 'yes', 'true']:
            config[key] = True
        elif config[key].lower() in ['n', 'f', '0', 'no', 'false']:
            config[key] = False
        else:
            raise Exception("Can't parse boolean argument " + key)

    return config



""" Helper module for kbuildparse."""

# Copyright (C) 2014-2015 Andreas Ruprecht <andreas.ruprecht@fau.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import cvehound.kbuildparse.data_structures as DataStructures

def build_precondition(input_list, additional=None):
    """ Build a DataStructures.Precondition object from a given @input_list.
    Additional constraints from @additional are added to the Precondition."""
    alternatives = []
    for alternative in input_list:
        string = " & ".join(alternative)
        if string != "":
            alternatives.append(string)
        else:
            # This case means that at least one unconditional path was found ->
            # return no condition.
            alternatives = []
            break

    alt_string = " | ".join(alternatives)

    ret = DataStructures.Precondition()
    if additional:
        for condition in additional:
            ret.add_condition(condition, keep_duplicates=False)

    if len(alternatives) > 1:
        ret.add_condition("(" + alt_string + ")", keep_duplicates=False)
    elif len(alt_string) > 1:
        ret.add_condition(alt_string, keep_duplicates=False)

    return ret

def guess_source_for_target(target):
    """
    for the given target, try to determine its source file.
    generic version for linux and busybox
    return None if no source file could be found
    """
    for suffix in ('.c', '.S', '.s', '.l', '.y', '.ppm'):
        sourcefile = target[:-2] + suffix
        if os.path.exists(sourcefile):
            return sourcefile
    return None

def remove_makefile_comment(line):
    """ Strips everything after the first # (Makefile comment) from a line."""
    return line.split("#", 1)[0].rstrip()

def get_multiline_from_file(infile):
    """ Reads a line from infile. If the line ends with a line continuation,
    it is substituted with a space and the next line is appended. Returns
    (True, line) if reading has succeeded, (False, "") otherwise. The boolean
    value is required to distinguish an error from empty lines in the input
    (which might also occur by stripping the comment from a line which only
    contains that comment)."""
    line = ""
    current = infile.readline()
    if not current:
        return (False, "")
    current = remove_makefile_comment(current)
    while current.endswith('\\'):
        current = current.replace('\\', ' ')
        line += current
        current = infile.readline()
        if not current:
            break
        current = remove_makefile_comment(current)
    line += current
    line.rstrip()
    return (True, line)

def get_config_string(item, model=None):
    """ Return a string with CONFIG_ for a given item. If the item is
    a tristate symbol in model, CONFIG_$(item)_MODULE is added as an
    alternative."""
    if item.startswith("CONFIG_"):
        item = item[7:]
    if model and model.get_type(item) == "tristate":
        return "(CONFIG_" + item + " || CONFIG_" + item + "_MODULE)"
    return "CONFIG_" + item



#!/usr/bin/env python3

import os
import re
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def find_version(source):
    version_file = read(source)
    version_match = re.search(r"^__VERSION__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

test_deps = [
    'pytest',
    'gitpython',
    'psutil'
]
extras = {
    'tests': test_deps,
}

setup(
    name='cvehound',
    version=find_version('cvehound/__init__.py'),
    author='Denis Efremov',
    author_email='efremov@linux.com',
    url='http://github.com/evdenis/cvehound',
    description='A tool to check linux kernel source dump for known CVEs',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    python_requires='>=3.5',
    install_requires=['sympy', 'lxml'],
    tests_require=test_deps,
    extras_require=extras,
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX :: Linux',
        "Operating System :: MacOS :: MacOS X",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    packages=[
        'cvehound',
        'cvehound.kbuildparse'
    ],
    license='GPLv3',
    keywords=['cve', 'linux', 'kernel', 'spatch', 'cve-scanning', 'coccinelle'],
    entry_points={
        'console_scripts': [
            'cvehound=cvehound.__main__:main',
            'cvehound_update_metadata=cvehound.scripts.update_metadata:main',
            'cvehound_update_rules=cvehound.scripts.update_rules:main'
        ]
    },
    include_package_data=True
)



#!/usr/bin/env python3

import pytest
from cvehound.exception import UnsupportedVersion

@pytest.mark.slow
def test_on_init(hound, repo, cve):
    fixes = hound.get_rule_fixes(cve)

    detect = False
    if fixes == 'v2.6.12-rc2' or \
       fixes == '1da177e4c3f41524e886b7f1b8a0c1fc7321cac2':
        detect = True
    repo.git.checkout('--force', 'v2.6.12-rc2')
    try:
        if detect:
            assert hound.check_cve(cve), cve + ' on first commit'
        else:
            assert not hound.check_cve(cve), cve + ' on first commit'
    except UnsupportedVersion:
        pytest.skip('Unsupported spatch version')



#!/usr/bin/env python3

import pytest
from conftest import missing_backports
from cvehound.exception import UnsupportedVersion

@pytest.mark.fast
@pytest.mark.notbackported(
    ('cve', 'branch'),
    missing_backports
)
def test_on_branch(hound, branch, cve):
    try:
        assert not hound.check_cve(cve), cve + ' on ' + branch
    except UnsupportedVersion:
        pytest.skip('Unsupported spatch version')



#!/usr/bin/env python3

import re
import pytest
from cvehound.cwe import CWE

def test_metadata(hound, cve):
    meta = hound.get_rule_metadata(cve)

    assert 'files' in meta, 'no "Files:" tag in the rule'
    assert 'fix' in meta, 'no "Fix:" tag in the rule'
    assert 'fixes' in meta, 'no "Fixes:" or "Detect-To:" tag in the rule'

    rule = hound.get_rule(cve)
    if rule.endswith('.grep'):
        return

    found = False
    cve_id = re.compile(r'CVE-\d{4}-\d{4,7}')
    with open(rule, 'rt') as fh:
        for line in fh:
            res = cve_id.search(line)
            if res:
                assert res.group(0) == cve, 'wrong CVE-id in the rule'
                found = True

    assert found, 'no CVE-id in the rule'
    assert hound.get_cve_metadata(cve), 'no metadata in kernel_cves.json'

def test_cwe():
    for cwe in CWE:
        if cwe == 'Other' or cwe == 'Unspecified':
            continue
        assert CWE[cwe], 'No CWE-id for "{}"'.format(cwe)

def test_cves_metadata_cwe(hound):
    meta = hound.metadata
    for cve in meta:
        if 'cwe' in meta[cve]:
            assert meta[cve]['cwe'] in CWE, 'Unknown CWE description "{}"'.format(meta[cve]['cwe'])

@pytest.mark.ownfixes(
    ('cve', 'reason'),
    [
        ('CVE-2021-0605',  "limited SA dump is not implemented in Linux-2.6.12-rc2"),
        ('CVE-2020-27825', "wrong fixes tag, see https://lore.kernel.org/linux-arm-msm/20210121140951.2a554a5e@gandalf.local.home/"),
        ('CVE-2020-14386', "wrong fixes tag, see https://seclists.org/oss-sec/2020/q3/150"),
        ('CVE-2019-15924', "wrong fixes tag, create_workqueue also can return NULL"),
        ('CVE-2021-20265', "wrong fixes tag, see https://lkml.org/lkml/2016/2/24/1054"),
        ('CVE-2015-8961', "wrong fixes tag, the error was introduced in 9d5065940693"),
        ('CVE-2017-12188', "wrong fixes tag, see https://www.spinics.net/lists/kvm/msg156651.html"),
        ('CVE-2017-7558', "wrong fixes tag, 52c52a61a39f intoduces it a bit earlier"),
        ('CVE-2016-9919', "wrong fixes tag, see https://bugzilla.redhat.com/show_bug.cgi?id=1403260"),
        ('CVE-2019-18809', "wrong fixes tag (too far)"),
        ('CVE-2019-19051', "wrong fixes tag because the fix fixing the fix fixing the memory leak"),
        ('CVE-2021-3635', "wrong fixes tag, commit fixes not only flowtables but also objs"),
        ('CVE-2022-3170', "CVE fix consists of 2 commits, 2nd commit fixes 1st one"),
        ('CVE-2024-0193', "wrong fixes tag, 5f68718b34a5 fixes race"),
    ]
)
def test_fixes(hound, repo, cve):
    cve_fix = hound.get_rule_fix(cve)
    cve_fixes = repo.git.rev_parse('--verify', hound.get_rule_fixes(cve) + '^{commit}')
    cve_fixes = cve_fixes[0:12]

    msg = repo.git.show('-s', '--format=%s\n%b', cve_fix)
    msg_fixes = list(map(lambda x: repo.git.rev_parse('--verify', x)[0:12],
                         re.findall(r'Fixes:\s*([0-9a-fA-F]{7,40})', msg)))
    if msg_fixes:
        if len(msg_fixes) == 1:
            msg_fixes = msg_fixes[0]
        assert cve_fixes in msg_fixes, \
            "{} vs {}".format(cve_fixes[0:12], msg_fixes)

def test_cve_disputed(hound, cve):
    meta = hound.get_cve_metadata(cve)
    rule = hound.cve_all_rules[cve]
    if 'nvd_text' in meta and not 'disputed' in rule:
        assert ' DIS' not in meta['nvd_text'], "{} DISPUTED".format(cve)

def test_cve_rejected(hound, cve):
    meta = hound.get_cve_metadata(cve)
    if 'nvd_text' in meta:
        assert ' REJ' not in meta['nvd_text'], "{} REJECTED".format(cve)

@pytest.mark.lkc
def test_cves_metadata_fix(hound, cve):
    fix = hound.get_rule_fix(cve)
    lkc_fix = hound.get_cve_metadata(cve)['fixes']
    assert fix == lkc_fix, "{} vs. {}".format(fix[0:12], lkc_fix[0:12])

@pytest.mark.lkc
def test_cves_metadata_fixes(hound, cve):
    fixes = hound.get_rule_fixes(cve)
    lkc_fixes = hound.get_cve_metadata(cve)['breaks']
    if fixes == 'v2.6.12-rc2':
        fixes = '1da177e4c3f41524e886b7f1b8a0c1fc7321cac2'
    assert fixes == lkc_fixes, "{} vs. {}".format(fixes[0:12], lkc_fixes[0:12])

@pytest.mark.lkc
def test_cves_metadata_fix_all(hound, repo):
    broken = []
    meta = hound.metadata
    for cve in meta:
        data = meta[cve]
        if 'fixes' not in data:
            continue
        if 'vendor_specific' in data and data['vendor_specific']:
            continue

        fix = data['fixes']
        if not fix:
            continue
        if not re.match(r'[0-9a-fA-F]{7,40}', fix):
            continue

        try:
            repo.git.rev_parse('--verify', fix + '^{commit}')
        except Exception:
            broken.append(cve)
    assert not broken, broken

@pytest.mark.lkc
def test_cves_metadata_fixes_all(hound, repo):
    broken = []
    meta = hound.metadata
    for cve in meta:
        data = meta[cve]
        if 'breaks' not in data:
            continue

        fixes = data['breaks']
        if not fixes:
            continue
        if not re.match(r'[0-9a-fa-f]{7,40}', fixes):
            continue

        try:
            repo.git.rev_parse('--verify', fixes + '^{commit}')
        except Exception:
            broken.append(cve)
    assert not broken, broken

@pytest.mark.lkc
def test_cves_metadata_fixes_all_git(hound, repo):
    broken = []
    meta = hound.metadata
    for cve in meta:
        data = meta[cve]
        if 'breaks' not in data:
            continue

        fixes = data['breaks']
        if not fixes:
            continue
        if not re.match(r'[0-9a-fa-f]{7,40}', fixes):
            continue

        try:
            fixes = repo.git.rev_parse('--verify', fixes + '^{commit}')
            fixes = fixes[0:12]
            msg = repo.git.show('-s', '--format=%s\n%b', fixes)
            msg_fixes = list(map(lambda x: repo.git.rev_parse('--verify', x)[0:12],
                             re.findall(r'Fixes:\s*([0-9a-fA-F]{7,40})', msg)))
        except Exception:
            continue

        if msg_fixes:
            if len(msg_fixes) == 1:
                msg_fixes = msg_fixes[0]
            if fixes not in msg_fixes:
                broken.append(cve)

    assert not broken, broken

@pytest.mark.lkc
def test_cves_metadata_title(hound, repo):
    broken = []
    meta = hound.metadata
    for cve in meta:
        data = meta[cve]

        if 'cmt_msg' not in data:
            continue
        data_msg = data['cmt_msg']

        fix = data['fixes']
        if not fix:
            continue
        if not re.match(r'[0-9a-fa-f]{7,40}', fix):
            continue

        try:
            fix = repo.git.rev_parse('--verify', fix + '^{commit}')
            fix = fix[0:12]
            git_msg = repo.git.show('-s', '--format=%s', fix)
        except Exception:
            continue

        if data_msg != git_msg:
            broken.append(cve)

    assert not broken, broken



#!/usr/bin/env python3

import os
import sys
import pkg_resources
import gzip
import json
import subprocess
import ssl
from urllib.request import urlopen, Request
from datetime import datetime
import lxml.etree as etree
from io import BytesIO
from zipfile import ZipFile

def get_exploit_status_from_fstec():
    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE
    req = Request('https://bdu.fstec.ru/files/documents/vulxml.zip', headers={'User-Agent': 'Mozilla/5.0'})
    with urlopen(req, context=ctx) as uh:
        with ZipFile(BytesIO(uh.read())) as zh:
            with zh.open('export/export.xml') as fh:
                parser = etree.XMLParser(recover=True)
                tree = etree.parse(fh, parser)

    public = set()
    private = set()
    for item in tree.xpath('//vul'):
        bdu_id = item.xpath('identifier/text()')[0]
        cve_id = None
        for vuln_id in item.xpath('identifiers/identifier'):
            if 'CVE' == vuln_id.get('type'):
                cve_id = vuln_id.text
                break
        is_linux = False
        for name in item.xpath('vulnerable_software/soft/name/text()'):
            if name == 'Linux' or name == 'linux':
                is_linux = True
        if not is_linux:
            continue
        if not cve_id:
            continue

        exploit_status = item.xpath('exploit_status/text()')[0]
        if 'открыт' in exploit_status: # 'открытом' == 'public'
            public.add(cve_id)
        elif 'уществует' in exploit_status: # == exists
            private.add(cve_id)

    return public, private

def get_commit_date(repo, commit):
    return int(subprocess.check_output(
            ['git', 'show', '-s', '--format=%ct', commit], cwd=repo, stderr=subprocess.DEVNULL, universal_newlines=True
    ).strip())

def main(args=sys.argv):
    if len(args) < 2 or not os.path.isdir(os.path.join(args[1], '.git')):
        print('Usage: {} <kernel_repo_dir> [metadata_file]'.format(args[0]), file=sys.stderr)
        sys.exit(1)
    repo = args[1]

    filename = None
    if len(args) == 3:
        filename = args[2]
    else:
        filename = os.environ.get('CVEHOUND_METADATA',
                                  pkg_resources.resource_filename('cvehound', 'data/kernel_cves.json.gz'))

    public, private = get_exploit_status_from_fstec()

    with urlopen('https://github.com/nluedtke/linux_kernel_cves/raw/master/data/kernel_cves.json') as fh:
        js = json.loads(fh.read().decode('utf-8'))

    # Corrupted data https://github.com/nluedtke/linux_kernel_cves/pull/379
    js = dict(filter(lambda x: x[0].startswith('CVE-'), js.items()))

    for cve, info in js.items():
        fix = info.get('fixes', '')
        if fix and repo:
            try:
                info['fix_date'] = get_commit_date(repo, fix)
            except Exception:
                pass
        info['exploit'] = cve in public or cve in private

    with gzip.open(filename, 'wt', encoding='utf-8') as fh:
        json.dump(js, fh)

if __name__ == '__main__':
    main(sys.argv)
